"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Container {
    constructor(providers = [], services = {}) {
        this.providers = providers;
        this.services = services;
        this.injectProperties = {};
    }
    register(...serviceClasses) {
        this.providers.push(...serviceClasses);
        serviceClasses.forEach((serviceClass) => {
            let service = serviceClass;
            if (service instanceof Function) {
                service = new serviceClass();
            }
            this.add(service);
        });
    }
    registerByName(name, serviceClass) {
        let service = serviceClass;
        if (service instanceof Function) {
            service = new serviceClass();
        }
        this.addByName(name, service);
    }
    get(name) {
        return this.services[name];
    }
    getByType(c) {
        return this.get(c.name.toLowerCase());
    }
    getAll() {
        return Object.assign({}, this.services);
    }
    addInjectProperty(target, propertyKey, serviceName = propertyKey) {
        if (target) {
            this.getInjectProperty(serviceName.toLowerCase()).push({ target, propertyKey, serviceName });
            this.inject(target, propertyKey, this.services[serviceName.toLowerCase()]);
        }
    }
    add(service) {
        return this.addByName(service.constructor.name, service);
    }
    addByName(name, service) {
        this.services[name] = service;
        this.services[name.toLowerCase()] = service;
        this.injectIntoProperties(service, name);
        return this.get(name);
    }
    inject(target, propertyKey, service) {
        if (target && service) {
            target[propertyKey] = service;
        }
    }
    injectIntoProperties(service, name = service.constructor.name) {
        this.getInjectProperty(name.toLowerCase()).forEach((property) => {
            this.inject(property.target, property.propertyKey, service);
        });
    }
    getInjectProperty(key) {
        if (!this.injectProperties[key]) {
            this.injectProperties[key] = [];
        }
        return this.injectProperties[key];
    }
}
exports.Container = Container;
exports.default = Container;
//# sourceMappingURL=Container.js.map